@@ -4,10 +4,12 @@ from stat import *
 import sys
 import os
 import platform
+from importlib.util import find_spec
 from making_with_code_cli.mwc_accounts_api import MWCAccountsAPI
 from making_with_code_cli.styles import (
     address,
     question,
+    debug,
     confirm,
     info,
     success,
@@ -41,6 +43,11 @@ INTRO_NOTES = [
 ]
 WORK_DIR_PERMISSIONS = S_IRWXU | S_IXGRP | S_IXOTH
 
+class PlatformNotSupported(Exception):
+    def __init__(self):
+        err = "This platform is not supported."
+        return super().__init__(err)
+
 class Platform(Flag):
     MAC = auto()
     UBUNTU = auto()
@@ -49,17 +56,34 @@ class Platform(Flag):
     UNSUPPORTED = 0
 
     @classmethod
-    def detect(self):
+    def detect(cls):
         if os.environ.get('REPL_ID'):
-            return self.REPLIT
+            return cls.REPLIT
         system_name = platform.system()
         if system_name == "Darwin":
-            return self.MAC
+            return cls.MAC
         if system_name == "Linux" and platform.freedesktop_os_release().get('NAME'):
             release_name = (platform.freedesktop_os_release() or {}).get('NAME')
             if release_name == "Ubuntu":
-                return self.UBUNTU
-        return self.UNSUPPORTED
+                return cls.UBUNTU
+        return cls.UNSUPPORTED
+
+    @classmethod
+    def package_manager(cls, brew_cask=False):
+        """Returns the command for the platform's package manager.
+        """
+        platform = cls.detect()
+        if platform == cls.MAC:
+            if brew_cask: 
+                return "brew install --cask "
+            else:
+                return "brew install "
+        elif platform == cls.UBUNTU:
+            return "sudo apt install "
+        elif platform == cls.REPLIT:
+            return "nix-env -iA nixpkgs."
+        else:
+            raise cls.NotSupported()
 
 def default_work_dir():
     if (Path.home() / "Desktop").exists():
@@ -189,17 +213,17 @@ class SetupTask:
     platform = Platform.SUPPORTED
     description = "A task"
 
-    def __init__(self, settings):
+    def __init__(self, settings, debug=False):
         self.settings = settings
+        self.debug = debug
 
     def run_task_if_needed(self):
-        if not Platform.detect() & self.platform:
-            return 
-        elif self.is_complete():
-            self.report_not_needed()
-        else:
-            self.run_task()
-            self.report_complete()
+        if Platform.detect() & self.platform:
+            if self.is_complete():
+                self.report_not_needed()
+            else:
+                self.run_task()
+                self.report_complete()
         
     def is_complete(self):
         return True    
@@ -213,6 +237,10 @@ class SetupTask:
     def report_complete(self):
         click.echo(success(f"{self.description} is complete!"))
 
+    def debug_log(self, message):
+        if self.debug:
+            click.echo(debug(message))
+
     def executable_on_path(self, name):
         return bool(run(f"which {name}", shell=True, capture_output=True).stdout)
 
@@ -225,7 +253,9 @@ class MWCShellConfig(SetupTask):
         return f"source {get_mwc_rc_path()}" in platform_rc_file().read_text()
 
     def run_task(self):
-        with platform_rc_file().open('a') as fh:
+        rc_file = platform_rc_file()
+        self.debug_log(f"Adding `mwc` path to {rc_file}")
+        with rc_file.open('a') as fh:
             result = run("which mwc", shell=True, check=True, text=True, capture_output=True)
             mwcpath = Path(result.stdout).parent
             fh.write(f"\n# MAKING WITH CODE\n")
@@ -294,7 +324,10 @@ class WriteShellConfig(SetupTask):
 
     def run_task(self):
         click.echo(address("Writing the MWC shell configuration file..."))
-        get_mwc_rc_path().write_text(self.generate_shell_config())
+        mwc_rc_path = get_mwc_rc_path()
+        config_text = self.generate_shell_config()
+        self.debug_log(f"Writing to {mwc_rc_path}:\n\n{config_text}\n\n")
+        mwc_rc_path.write_text(config_text)
         run(f"source {get_mwc_rc_path()}", shell=True, check=True)
 
     def generate_shell_config(self):
@@ -312,52 +345,53 @@ class WriteShellConfig(SetupTask):
         return f
 
 class InstallPackage(SetupTask):
-    package_name = "package"
+    """A subclass of SetupTask for packages to be installed by the Platform package manager.
+    """
+    executable_name = "package"
     brew_name = "package"
     apt_name = "package"
     nix_name = "package"
-    cask = False
+    brew_cask = False
 
     def __init__(self, *args, **kwargs):
-        self.description = f"Install {self.package_name}"
+        self.description = f"Install {self.executable_name}"
         super().__init__(*args, **kwargs)
 
     def is_complete(self):
-        return self.executable_on_path(self.package_name)
+        return self.executable_on_path(self.executable_name)
+
+    def get_package_name(self):
+        system_platform = Platform.detect()
+        if system_platform == Platform.MAC:
+            return self.brew_name
+        elif system_platform == Platform.UBUNTU:
+            return self.apt_name
+        elif system_platform == Platform.REPLIT:
+            return self.nix_name
+        else:
+            raise PlatformNotSupported()
     
     def run_task(self):
-        platform = Platform.detect()
-        click.echo(address(f"Installing {self.package_name}..."))
-        if platform & Platform.MAC:
-            if self.cask:
-                run(f"brew install {self.brew_name}", shell=True, check=True)
-            else:
-                run(f"brew install --cask {self.brew_name}", shell=True, check=True)
-        elif platform & Platform.UBUNTU:
-            run(f"apt install {self.apt_name}", shell=True, check=True)
-        elif platform & Platform.REPLIT:
-            run(f"nix-env -iA nixpkgs.{self.nix_name}", shell=True, check=True)
-        else:
-            raise IOError(f"Platform {platform} not supported...")
+        click.echo(address(f"Installing {self.executable_name}..."))
+        package_manager = Platform.package_manager(brew_cask=self.brew_cask)
+        package_name = self.get_package_name()
+        self.debug_log(f"Running: {package_manager}{package_name}")
+        run(f"{package_manager}{package_name}", shell=True, check=True)
 
 class InstallCurl(InstallPackage):
     platform = Platform.UBUNTU
-    package_name = brew_name = apt_name = "curl"
-
-class InstallPython3(InstallPackage):
-    platform = Platform.MAC | Platform.UBUNTU
-    package_name = brew_name = apt_name = "python3"
+    executable_name = apt_name = "curl"
 
 class InstallGit(InstallPackage):
     platform = Platform.MAC | Platform.UBUNTU
-    package_name = brew_name = apt_name = "git"
+    executable_name = brew_name = apt_name = "git"
 
 class InstallTree(InstallPackage):
-    package_name = brew_name = apt_name = nix_name = "tree"
+    executable_name = brew_name = apt_name = nix_name = "tree"
 
 class InstallVSCode(InstallPackage):
     platform = Platform.MAC | Platform.UBUNTU
-    package_name = "code"
+    executable_name = "code"
     brew_name = "visual-studio-code"
     cask = True
 
@@ -369,24 +403,21 @@ class InstallVSCode(InstallPackage):
             return super().run_task()
 
 class InstallImageMagick(InstallPackage):
-    package_name = "magick"
+    executable_name = "magick"
     brew_name = apt_name = nix_name = "imagemagick"
 
+    def is_complete(self):
+        return self.executable_on_path("magick") or self.executable_on_path("convert")
+
 class InstallHttpie(InstallPackage):
-    package_name = "http"
+    executable_name = "http"
     brew_name = apt_name = nix_name = "httpie"
 
 class InstallScipy(InstallPackage):
-    package_name = brew_name = apt_name = "scipy"
+    executable_name = brew_name = apt_name = nix_name = "scipy"
 
     def is_complete(self):
-        return True
-        try:
-            import scipy
-            return True
-        except ModuleNotFoundError as e:
-            print(e)
-            return False
+        return find_spec("scipy") is not None
 
 class GitConfiguration(SetupTask):
     """Configure global git settings.
@@ -437,5 +468,3 @@ class GitConfiguration(SetupTask):
         return git_config
 
 
-
-
